@@ -1,3 +1,4 @@
+use crate::directory::Directory;
 use crate::error::Error;
 use crate::manifest::Edition;
 use serde::de::value::MapAccessDeserializer;
@@ -6,30 +7,33 @@ use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::collections::BTreeMap as Map;
 use std::fmt;
 use std::fs;
-use std::path::Path;
 use std::path::PathBuf;
 use toml::Value;
 
-pub fn get_manifest(manifest_dir: &Path) -> Manifest {
+pub fn get_manifest(manifest_dir: &Directory) -> Manifest {
     try_get_manifest(manifest_dir).unwrap_or_default()
 }
 
-fn try_get_manifest(manifest_dir: &Path) -> Result<Manifest, Error> {
+fn try_get_manifest(manifest_dir: &Directory) -> Result<Manifest, Error> {
     let cargo_toml_path = manifest_dir.join("Cargo.toml");
     let manifest_str = fs::read_to_string(cargo_toml_path)?;
     let mut manifest: Manifest = toml::from_str(&manifest_str)?;
 
     fix_dependencies(&mut manifest.dependencies, manifest_dir);
     fix_dependencies(&mut manifest.dev_dependencies, manifest_dir);
+    for target in manifest.target.values_mut() {
+        fix_dependencies(&mut target.dependencies, manifest_dir);
+        fix_dependencies(&mut target.dev_dependencies, manifest_dir);
+    }
 
     Ok(manifest)
 }
 
-pub fn get_workspace_manifest(manifest_dir: &Path) -> WorkspaceManifest {
+pub fn get_workspace_manifest(manifest_dir: &Directory) -> WorkspaceManifest {
     try_get_workspace_manifest(manifest_dir).unwrap_or_default()
 }
 
-pub fn try_get_workspace_manifest(manifest_dir: &Path) -> Result<WorkspaceManifest, Error> {
+pub fn try_get_workspace_manifest(manifest_dir: &Directory) -> Result<WorkspaceManifest, Error> {
     let cargo_toml_path = manifest_dir.join("Cargo.toml");
     let manifest_str = fs::read_to_string(cargo_toml_path)?;
     let mut manifest: WorkspaceManifest = toml::from_str(&manifest_str)?;
@@ -40,14 +44,14 @@ pub fn try_get_workspace_manifest(manifest_dir: &Path) -> Result<WorkspaceManife
     Ok(manifest)
 }
 
-fn fix_dependencies(dependencies: &mut Map<String, Dependency>, dir: &Path) {
+fn fix_dependencies(dependencies: &mut Map<String, Dependency>, dir: &Directory) {
     dependencies.remove("trybuild");
     for dep in dependencies.values_mut() {
-        dep.path = dep.path.as_ref().map(|path| dir.join(path));
+        dep.path = dep.path.as_ref().map(|path| Directory::new(dir.join(path)));
     }
 }
 
-fn fix_patches(patches: &mut Map<String, RegistryPatch>, dir: &Path) {
+fn fix_patches(patches: &mut Map<String, RegistryPatch>, dir: &Directory) {
     for registry in patches.values_mut() {
         registry.crates.remove("trybuild");
         for patch in registry.crates.values_mut() {
@@ -56,7 +60,7 @@ fn fix_patches(patches: &mut Map<String, RegistryPatch>, dir: &Path) {
     }
 }
 
-fn fix_replacements(replacements: &mut Map<String, Patch>, dir: &Path) {
+fn fix_replacements(replacements: &mut Map<String, Patch>, dir: &Directory) {
     replacements.remove("trybuild");
     for replacement in replacements.values_mut() {
         replacement.path = replacement.path.as_ref().map(|path| dir.join(path));
@@ -81,6 +85,8 @@ pub struct Manifest {
     pub dependencies: Map<String, Dependency>,
     #[serde(default, alias = "dev-dependencies")]
     pub dev_dependencies: Map<String, Dependency>,
+    #[serde(default)]
+    pub target: Map<String, TargetDependencies>,
 }
 
 #[derive(Deserialize, Default, Debug)]
@@ -95,7 +101,7 @@ pub struct Dependency {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub version: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub path: Option<PathBuf>,
+    pub path: Option<Directory>,
     #[serde(
         rename = "default-features",
         default = "get_true",
@@ -104,10 +110,30 @@ pub struct Dependency {
     pub default_features: bool,
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub features: Vec<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub git: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub branch: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub tag: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub rev: Option<String>,
     #[serde(flatten)]
     pub rest: Map<String, Value>,
 }
 
+#[derive(Serialize, Deserialize, Clone, Debug)]
+pub struct TargetDependencies {
+    #[serde(default, skip_serializing_if = "Map::is_empty")]
+    pub dependencies: Map<String, Dependency>,
+    #[serde(
+        default,
+        alias = "dev-dependencies",
+        skip_serializing_if = "Map::is_empty"
+    )]
+    pub dev_dependencies: Map<String, Dependency>,
+}
+
 #[derive(Serialize, Deserialize, Clone, Debug)]
 #[serde(transparent)]
 pub struct RegistryPatch {
@@ -122,6 +148,12 @@ pub struct Patch {
     pub git: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub branch: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub tag: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub rev: Option<String>,
+    #[serde(flatten)]
+    pub rest: Map<String, Value>,
 }
 
 fn get_true() -> bool {
@@ -167,6 +199,10 @@ impl<'de> Deserialize<'de> for Dependency {
                     path: None,
                     default_features: true,
                     features: Vec::new(),
+                    git: None,
+                    branch: None,
+                    tag: None,
+                    rev: None,
                     rest: Map::new(),
                 })
             }
