@@ -1,4 +1,4 @@
-use crate::dependencies::{Dependency, Patch, RegistryPatch};
+use crate::dependencies::{Dependency, Patch, RegistryPatch, TargetDependencies};
 use serde::{Deserialize, Serialize};
 use std::collections::BTreeMap as Map;
 use std::ffi::OsStr;
@@ -10,6 +10,8 @@ pub struct Manifest {
     #[serde(skip_serializing_if = "Map::is_empty")]
     pub features: Map<String, Vec<String>>,
     pub dependencies: Map<String, Dependency>,
+    #[serde(skip_serializing_if = "Map::is_empty")]
+    pub target: Map<String, TargetDependencies>,
     #[serde(rename = "bin")]
     pub bins: Vec<Bin>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -34,6 +36,8 @@ pub enum Edition {
     E2015,
     #[serde(rename = "2018")]
     E2018,
+    #[serde(rename = "2021")]
+    E2021,
 }
 
 #[derive(Serialize, Debug)]
